//==================================================================
//
// File....... Demo_I2C.c
// Compiler... CodeVisionAVR
// Revision... $Revision: 1.00 $
// Purpose.... Demo Code for US and IR Ranger
// Author..... Innovative Electronics, Inc.
// E-mail..... support@innovativeelectronics.com
// Started....
// Updated.... 29 FEB 2007
//==================================================================

// -----[ Program Description ]-------------------------------------
//
// This program demonstrates the use of US and IR Ranger 
// Module with I2C-Bus interface and then send the measurement 
// results to UART Serial Port. 
//
// Three distances measurement is done each second through an 
// ultrasonic ranger, and two infrared ranger connected to the Smart 
// Sense module. The output is three sets of four ASCII character 
// digits representing the range in millimeters (mm), separated by 
// space (ASCII 20), and followed by carriage return (ASCII 13).
// The baudrate 9600, 8 bits, no parity, with one stop bit.
//
// Smart Sense module address is set to 0xEE
// Chip type           : AT90S2313
// Clock frequency     : 4.000000 MHz
// SDA pin             : PORTB.0
// SCL pin             : PORTB.1
// RDY/BUSY pin        : PINB.3

// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the AT90S2313


#pragma used+
sfrb ACSR=8;
sfrb UBRR=9;
sfrb UCR=0xa;
sfrb USR=0xb;
sfrb UDR=0xc;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb WDTCR=0x21;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb OCR1L=0x2a;
sfrb OCR1H=0x2b;
sfrw OCR1=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb GIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-


// Interrupt vectors definitions


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x10
	.SET power_ctrl_reg=mcucr
	#endif
#endasm


/*
  CodeVisionAVR C Compiler
  (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.

  Prototypes for I2C bus master functions

  BEFORE #include -ING THIS FILE YOU
  MUST DECLARE THE I/O ADDRESS OF THE
  DATA REGISTER OF THE PORT AT WHICH
  THE I2C BUS IS CONNECTED AND
  THE DATA BITS USED FOR SDA & SCL

  EXAMPLE FOR PORTB:

    #asm
        .equ __i2c_port=0x18
        .equ __sda_bit=3
        .equ __scl_bit=4
    #endasm
    #include <i2c.h>
*/


#pragma used+
void i2c_init(void);
unsigned char i2c_start(void);
void i2c_stop(void);
unsigned char i2c_read(unsigned char ack);
unsigned char i2c_write(unsigned char data);
#pragma used-


// CodeVisionAVR C Compiler
// (C) 1998-2006 Pavel Haiduc, HP InfoTech S.R.L.

// Prototypes for standard I/O functions


// CodeVisionAVR C Compiler
// (C) 1998-2002 Pavel Haiduc, HP InfoTech S.R.L.

// Variable length argument list macros



typedef char *va_list;





#pragma used+

char getchar(void);
void putchar(char c);
void puts(char *str);
void putsf(char flash *str);

char *gets(char *str,unsigned char len);

void printf(char flash *fmtstr,...);
void sprintf(char *str, char flash *fmtstr,...);
void snprintf(char *str, unsigned char size, char flash *fmtstr,...);
void vprintf (char flash * fmtstr, va_list argptr);
void vsprintf (char *str, char flash * fmtstr, va_list argptr);
void vsnprintf (char *str, unsigned char size, char flash * fmtstr, va_list argptr);
signed char scanf(char flash *fmtstr,...);
signed char sscanf(char *str, char flash *fmtstr,...);




                                   

            


#pragma used-

#pragma library stdio.lib


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-


typedef unsigned char  uint8_t;
typedef unsigned int   uint16_t;

// I2C Bus functions
#asm
    .equ __i2c_port=0x18 ;PORTB
    .equ __sda_bit=0
    .equ __scl_bit=1
#endasm


uint16_t US,IR1,IR2;

void readReg(void)
{
    uint8_t k;
    i2c_start();
    i2c_write(0xEE);
    i2c_write(3);
    i2c_stop();

    i2c_start();
    i2c_write(0xEF);

    k=i2c_read(1);putchar(k);
    k=i2c_read(1);putchar(k);
    k=i2c_read(1);putchar(k);
    k=i2c_read(0);putchar(k);
    i2c_stop();
}

void writeReg(uint8_t cReg,uint8_t cUS,uint8_t cIR1,uint8_t cIR2)
{
    i2c_start();
    i2c_write(0xEE);
    i2c_write(2);
    i2c_write(cReg);
    i2c_write(cUS);
    i2c_write(cIR1);
    i2c_write(cIR2);
    i2c_stop();
}

void startRanging(void)
{
    i2c_start();
    i2c_write(0xEE);
    i2c_write(0);
    i2c_stop();
}

void readData(void)
{
    uint8_t k;
    i2c_start();
    i2c_write(0xEE);
    i2c_write(1);
    i2c_stop();

    i2c_start();
    i2c_write(0xEF);

    k=i2c_read(1);
    US = k * 256;
    k=i2c_read(1);
    US = US + k;

    k=i2c_read(1);
    IR1 = k * 256;
    k=i2c_read(1);
    IR1 = IR1 + k;

    k=i2c_read(1);
    IR2 = k * 256;
    k=i2c_read(0);
    IR2 = IR2 + k;

    i2c_stop();
}

void sendData(void)
{                                       

    putchar('U');
    putchar('S');
    putchar(':');
    putchar((US/1000)%10 + 0x30);
    putchar((US/100)%10  + 0x30);
    putchar((US/10)%10   + 0x30);
    putchar((US%10)      + 0x30);
    putchar('m');
    putchar('m');
    putchar(0x0D);

    putchar('1');
    putchar(':');
//    putchar(' ');
    putchar((IR1/1000)%10 + 0x30);
    putchar((IR1/100)%10  + 0x30);
    putchar((IR1/10)%10   + 0x30);
    putchar((IR1%10)      + 0x30);
    putchar('m');
    putchar('m');
    putchar(0x0D);

    putchar('2');
    putchar(':');
    putchar((IR2/1000)%10 + 0x30);
    putchar((IR2/100)%10  + 0x30);
    putchar((IR2/10)%10   + 0x30);
    putchar((IR2%10)      + 0x30);
    putchar('m');
    putchar('m');

    putchar(0x0D);
    putchar(0x0D);
}

void main(void)
{
// Input/Output Ports initialization
// Port B initialization
PORTB=0x08;
DDRB=0x00;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
// UART Baud rate: 9600
UCR=0x18;
UBRR=0x19;

// I2C Bus initialization
i2c_init();

delay_ms(1000);
writeReg(0x0F,128,128,128);
while (1)      
    {
        startRanging();

        while (PINB.3);//tunggu sampai high
        //        delay_ms(10);
        while (!PINB.3);  //tunggu jika masih low
//        delay_ms(100);

        readData();
        sendData();
        delay_ms(2);
    };
}
