//==================================================================
//
// File....... Demo_I2C.c
// Compiler... CodeVisionAVR
// Revision... $Revision: 1.00 $
// Purpose.... Demo Code for US and IR Ranger
// Author..... Innovative Electronics, Inc.
// E-mail..... support@innovativeelectronics.com
// Started....
// Updated.... 29 FEB 2007
//==================================================================

// -----[ Program Description ]-------------------------------------
//
// This program demonstrates the use of US and IR Ranger 
// Module with I2C-Bus interface and then send the measurement 
// results to UART Serial Port. 
//
// Three distances measurement is done each second through an 
// ultrasonic ranger, and two infrared ranger connected to the Smart 
// Sense module. The output is three sets of four ASCII character 
// digits representing the range in millimeters (mm), separated by 
// space (ASCII 20), and followed by carriage return (ASCII 13).
// The baudrate 9600, 8 bits, no parity, with one stop bit.
//
// Smart Sense module address is set to 0xEE
// Chip type           : AT90S2313
// Clock frequency     : 4.000000 MHz
// SDA pin             : PORTB.0
// SCL pin             : PORTB.1
// RDY/BUSY pin        : PINB.3

#include <90s2313.h>
#include <i2c.h>
#include <stdio.h>
#include <delay.h>

typedef unsigned char  uint8_t;
typedef unsigned int   uint16_t;

// I2C Bus functions
#asm
    .equ __i2c_port=0x18 ;PORTB
    .equ __sda_bit=0
    .equ __scl_bit=1
#endasm

#define ready   PINB.3

uint16_t US,IR1,IR2;

void readReg(void)
{
    uint8_t k;
    i2c_start();
    i2c_write(0xEE);
    i2c_write(3);
    i2c_stop();

    i2c_start();
    i2c_write(0xEF);

    k=i2c_read(1);putchar(k);
    k=i2c_read(1);putchar(k);
    k=i2c_read(1);putchar(k);
    k=i2c_read(0);putchar(k);
    i2c_stop();
}

void writeReg(uint8_t cReg,uint8_t cUS,uint8_t cIR1,uint8_t cIR2)
{
    i2c_start();
    i2c_write(0xEE);
    i2c_write(2);
    i2c_write(cReg);
    i2c_write(cUS);
    i2c_write(cIR1);
    i2c_write(cIR2);
    i2c_stop();
}

void startRanging(void)
{
    i2c_start();
    i2c_write(0xEE);
    i2c_write(0);
    i2c_stop();
}

void readData(void)
{
    uint8_t k;
    i2c_start();
    i2c_write(0xEE);
    i2c_write(1);
    i2c_stop();

    i2c_start();
    i2c_write(0xEF);

    k=i2c_read(1);
    US = k * 256;
    k=i2c_read(1);
    US = US + k;

    k=i2c_read(1);
    IR1 = k * 256;
    k=i2c_read(1);
    IR1 = IR1 + k;

    k=i2c_read(1);
    IR2 = k * 256;
    k=i2c_read(0);
    IR2 = IR2 + k;

    i2c_stop();
}

void sendData(void)
{                                       

    putchar('U');
    putchar('S');
    putchar(':');
    putchar((US/1000)%10 + 0x30);
    putchar((US/100)%10  + 0x30);
    putchar((US/10)%10   + 0x30);
    putchar((US%10)      + 0x30);
    putchar('m');
    putchar('m');
    putchar(0x0D);

    putchar('1');
    putchar(':');
//    putchar(' ');
    putchar((IR1/1000)%10 + 0x30);
    putchar((IR1/100)%10  + 0x30);
    putchar((IR1/10)%10   + 0x30);
    putchar((IR1%10)      + 0x30);
    putchar('m');
    putchar('m');
    putchar(0x0D);

    putchar('2');
    putchar(':');
    putchar((IR2/1000)%10 + 0x30);
    putchar((IR2/100)%10  + 0x30);
    putchar((IR2/10)%10   + 0x30);
    putchar((IR2%10)      + 0x30);
    putchar('m');
    putchar('m');

    putchar(0x0D);
    putchar(0x0D);
}

void main(void)
{
// Input/Output Ports initialization
// Port B initialization
PORTB=0x08;
DDRB=0x00;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
// UART Baud rate: 9600
UCR=0x18;
UBRR=0x19;

// I2C Bus initialization
i2c_init();

delay_ms(1000);
writeReg(0x0F,128,128,128);
while (1)      
    {
        startRanging();

        while (ready);//tunggu sampai high
        //        delay_ms(10);
        while (!ready);  //tunggu jika masih low
//        delay_ms(100);

        readData();
        sendData();
        delay_ms(100);
    };
}