//===============================================================
//
// File....... Demo_Pulse.c
// Compiler... CodeVisionAVR
// Revision... $Revision: 1.00 $
// Purpose.... Demo Code for US and IR Ranger
// Author..... Innovative Electronics, Inc.
// E-mail..... support@innovativeelectronics.com
// Started....
// Updated.... 29 FEB 2007
//==================================================================

// -----[ Program Description ]-------------------------------------
//
// This program demonstrates the use of US and IR Ranger 
// Module with PulseWidth interface and then send the measurement 
// results to UART Serial Port. 
//
// The distance measurement is done each second through an 
// ultrasonic ranger of the Smart Sense module. The output is four 
// ASCII character digits representing the range in millimeters (mm), 
// followed by carriage return (ASCII 13). The baudrate 9600, 8 bits, 
// no parity, with one stop bit.
//
// Smart Sense module address is set to 0xEE
// Chip type           : AT90S2313
// Clock frequency     : 4.000000 MHz
// SIG pin             : PORTB.2
// RDY/BUSY pin        : PINB.3

#include <90s2313.h>
#include <stdio.h>
#include <delay.h>

typedef unsigned int   uint16_t;

#define SIG_out     PORTB.2
#define SIG_in      PINB.2
#define SIG_dir     DDRB.2
#define ready       PINB.3

uint16_t US;

void sendData(void)
{
    putchar('J');
    putchar('a');
    putchar('r');
    putchar('a');
    putchar('k');
    putchar(':');
    putchar(' ');
    putchar((US/1000)%10 + 0x30 );
    putchar((US/100)%10  + 0x30 );
    putchar((US%100)/10  + 0x30 );
    putchar((US%10)      + 0x30 );
    putchar(' ');
    putchar('m');
    putchar('m');

    putchar(0x0D);
}

void main(void)
{
unsigned int counter;
// Input/Output Ports initialization

// Port B initialization
PORTB=0x0C;
DDRB=0x04;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
// UART Baud rate: 9600
UCR=0x18;
UBRR=0x19;

delay_ms(1000);
while (1)
    {
        SIG_dir  = 1;    // set SIG pin as output

        SIG_out  = 0;    //
        delay_us(50);    // send start pulse
        SIG_out  = 1;    //

        SIG_dir  = 0;    // set SIG pin as input

        TCNT1=0; 
        counter=0;
        while (SIG_in && counter<30000)     // wait for return pulse
        counter++;
        TCCR1B=0x02;        // start timer (timer period = 2uS)
        while ((!SIG_in) && !(TIFR&0x80)); //TIFRx80 :bit overflow:bit ini tidak                                            // selalu sama letaknya di setiap UC AVR
        TCCR1B=0x00;        // stop timer
        US = TCNT1;         // save timer value to variable US

        US = US/5;          // convert to milimeters

        sendData();
        delay_ms(100);
    };
}

/*
US=0; //TCCR1B=0x02;        // start timer (timer period = 2uS)
        while ((!SIG_in) && US<40000)
        {
        delay_us(70);
        US++;  
        }//1 us =0.1mm
        //US=US;    
            // timing the return pulse
        //TCCR1B=0x00;        // stop timer
        //US = TCNT1;         // save timer value to variable US

        US = US*7;          // c */